@@ -21,6 +21,7 @@ require "delegate"
 require "logstash/util/byte_value"
 require "logstash/util/substitution_variables"
 require "logstash/util/time_value"
+require "i18n"
 
 module LogStash
   class Settings
@@ -49,6 +50,8 @@ module LogStash
       "dead_letter_queue.enable",
       "dead_letter_queue.flush_interval",
       "dead_letter_queue.max_bytes",
+      "dead_letter_queue.storage_policy",
+      "dead_letter_queue.retain.age",
       "metric.collect",
       "pipeline.plugin_classloaders",
       "path.config",
@@ -73,7 +76,6 @@ module LogStash
       "queue.type",
     ]
 
-
     def initialize
       @settings = {}
       # Theses settings were loaded from the yaml file
@@ -121,7 +123,7 @@ module LogStash
       get_setting(setting_name).set?
     end
 
-    def clone
+    def clone(*args)
       get_subset(".*")
     end
     alias_method :dup, :clone
@@ -183,7 +185,7 @@ module LogStash
       @settings.values.each(&:reset)
     end
 
-    def from_yaml(yaml_path, file_name="logstash.yml")
+    def from_yaml(yaml_path, file_name = "logstash.yml")
       settings = read_yaml(::File.join(yaml_path, file_name))
       self.merge(deep_replace(flatten_hash(settings)), true)
       self
@@ -221,12 +223,12 @@ module LogStash
       YAML.safe_load(IO.read(path)) || {}
     end
 
-    def flatten_hash(h,f="",g={})
+    def flatten_hash(h, f = "", g = {})
       return g.update({ f => h }) unless h.is_a? Hash
       if f.empty?
-        h.each { |k,r| flatten_hash(r,k,g) }
+        h.each { |k, r| flatten_hash(r, k, g) }
       else
-        h.each { |k,r| flatten_hash(r,"#{f}.#{k}",g) }
+        h.each { |k, r| flatten_hash(r, "#{f}.#{k}", g) }
       end
       g
     end
@@ -237,7 +239,7 @@ module LogStash
 
     attr_reader :name, :default
 
-    def initialize(name, klass, default=nil, strict=true, &validator_proc)
+    def initialize(name, klass, default = nil, strict = true, &validator_proc)
       @name = name
       unless klass.is_a?(Class)
         raise ArgumentError.new("Setting \"#{@name}\" must be initialized with a class (received #{klass})")
@@ -341,7 +343,7 @@ module LogStash
     end
 
     class Coercible < Setting
-      def initialize(name, klass, default=nil, strict=true, &validator_proc)
+      def initialize(name, klass, default = nil, strict = true, &validator_proc)
         @name = name
         unless klass.is_a?(Class)
           raise ArgumentError.new("Setting \"#{@name}\" must be initialized with a class (received #{klass})")
@@ -375,7 +377,7 @@ module LogStash
     ### Specific settings #####
 
     class Boolean < Coercible
-      def initialize(name, default, strict=true, &validator_proc)
+      def initialize(name, default, strict = true, &validator_proc)
         super(name, Object, default, strict, &validator_proc)
       end
 
@@ -392,7 +394,7 @@ module LogStash
     end
 
     class Numeric < Coercible
-      def initialize(name, default=nil, strict=true)
+      def initialize(name, default = nil, strict = true)
         super(name, ::Numeric, default, strict)
       end
 
@@ -412,7 +414,7 @@ module LogStash
     end
 
     class Integer < Coercible
-      def initialize(name, default=nil, strict=true)
+      def initialize(name, default = nil, strict = true)
         super(name, ::Integer, default, strict)
       end
 
@@ -430,7 +432,7 @@ module LogStash
     end
 
     class PositiveInteger < Integer
-      def initialize(name, default=nil, strict=true)
+      def initialize(name, default = nil, strict = true)
         super(name, default, strict) do |v|
           if v > 0
             true
@@ -444,7 +446,7 @@ module LogStash
     class Port < Integer
       VALID_PORT_RANGE = 1..65535
 
-      def initialize(name, default=nil, strict=true)
+      def initialize(name, default = nil, strict = true)
         super(name, default, strict) { |value| valid?(value) }
       end
 
@@ -456,8 +458,8 @@ module LogStash
     class PortRange < Coercible
       PORT_SEPARATOR = "-"
 
-      def initialize(name, default=nil, strict=true)
-        super(name, ::Range, default, strict=true) { |value| valid?(value) }
+      def initialize(name, default = nil, strict = true)
+        super(name, ::Range, default, strict = true) { |value| valid?(value) }
       end
 
       def valid?(range)
@@ -485,13 +487,13 @@ module LogStash
 
       def validate(value)
         unless valid?(value)
-          raise ArgumentError.new("Invalid value \"#{value}, valid options are within the range of #{Port::VALID_PORT_RANGE.first}-#{Port::VALID_PORT_RANGE.last}")
+          raise ArgumentError.new("Invalid value \"#{name}: #{value}\", valid options are within the range of #{Port::VALID_PORT_RANGE.first}-#{Port::VALID_PORT_RANGE.last}")
         end
       end
     end
 
     class Validator < Setting
-      def initialize(name, default=nil, strict=true, validator_class=nil)
+      def initialize(name, default = nil, strict = true, validator_class = nil)
         @validator_class = validator_class
         super(name, ::Object, default, strict)
       end
@@ -502,7 +504,7 @@ module LogStash
     end
 
     class String < Setting
-      def initialize(name, default=nil, strict=true, possible_strings=[])
+      def initialize(name, default = nil, strict = true, possible_strings = [])
         @possible_strings = possible_strings
         super(name, ::String, default, strict)
       end
@@ -510,7 +512,7 @@ module LogStash
       def validate(value)
         super(value)
         unless @possible_strings.empty? || @possible_strings.include?(value)
-          raise ArgumentError.new("Invalid value \"#{value}\". Options are: #{@possible_strings.inspect}")
+          raise ArgumentError.new("Invalid value \"#{name}: #{value}\". Options are: #{@possible_strings.inspect}")
         end
       end
     end
@@ -523,7 +525,7 @@ module LogStash
     end
 
     class Password < Coercible
-      def initialize(name, default=nil, strict=true)
+      def initialize(name, default = nil, strict = true)
         super(name, LogStash::Util::Password, default, strict)
       end
 
@@ -542,12 +544,55 @@ module LogStash
       end
     end
 
+    class ValidatedPassword < Setting::Password
+      def initialize(name, value, password_policies)
+        @password_policies = password_policies
+        super(name, value, true)
+      end
+
+      def coerce(password)
+        if password && !password.kind_of?(::LogStash::Util::Password)
+          raise(ArgumentError, "Setting `#{name}` could not coerce LogStash::Util::Password value to password")
+        end
+
+        policies = build_password_policies
+        validatedResult = LogStash::Util::PasswordValidator.new(policies).validate(password.value)
+        if validatedResult.length() > 0
+          if @password_policies.fetch(:mode).eql?("WARN")
+            logger.warn("Password #{validatedResult}.")
+          else
+            raise(ArgumentError, "Password #{validatedResult}.")
+          end
+        end
+        password
+      end
+
+      def build_password_policies
+        policies = {}
+        policies[Util::PasswordPolicyType::EMPTY_STRING] = Util::PasswordPolicyParam.new
+        policies[Util::PasswordPolicyType::LENGTH] = Util::PasswordPolicyParam.new("MINIMUM_LENGTH", @password_policies.dig(:length, :minimum).to_s)
+        if @password_policies.dig(:include, :upper).eql?("REQUIRED")
+          policies[Util::PasswordPolicyType::UPPER_CASE] = Util::PasswordPolicyParam.new
+        end
+        if @password_policies.dig(:include, :lower).eql?("REQUIRED")
+          policies[Util::PasswordPolicyType::LOWER_CASE] = Util::PasswordPolicyParam.new
+        end
+        if @password_policies.dig(:include, :digit).eql?("REQUIRED")
+          policies[Util::PasswordPolicyType::DIGIT] = Util::PasswordPolicyParam.new
+        end
+        if @password_policies.dig(:include, :symbol).eql?("REQUIRED")
+          policies[Util::PasswordPolicyType::SYMBOL] = Util::PasswordPolicyParam.new
+        end
+        policies
+      end
+    end
+
     # The CoercibleString allows user to enter any value which coerces to a String.
     # For example for true/false booleans; if the possible_strings are ["foo", "true", "false"]
     # then these options in the config file or command line will be all valid: "foo", true, false, "true", "false"
     #
     class CoercibleString < Coercible
-      def initialize(name, default=nil, strict=true, possible_strings=[], &validator_proc)
+      def initialize(name, default = nil, strict = true, possible_strings = [], &validator_proc)
         @possible_strings = possible_strings
         super(name, Object, default, strict, &validator_proc)
       end
@@ -565,9 +610,9 @@ module LogStash
     end
 
     class ExistingFilePath < Setting
-      def initialize(name, default=nil, strict=true)
+      def initialize(name, default = nil, strict = true)
         super(name, ::String, default, strict) do |file_path|
-          if !::File.exists?(file_path)
+          if !::File.exist?(file_path)
             raise ::ArgumentError.new("File \"#{file_path}\" must exist but was not found.")
           else
             true
@@ -577,7 +622,7 @@ module LogStash
     end
 
     class WritableDirectory < Setting
-      def initialize(name, default=nil, strict=false)
+      def initialize(name, default = nil, strict = false)
         super(name, ::String, default, strict)
       end
 
@@ -622,8 +667,8 @@ module LogStash
     end
 
     class Bytes < Coercible
-      def initialize(name, default=nil, strict=true)
-        super(name, ::Integer, default, strict=true) { |value| valid?(value) }
+      def initialize(name, default = nil, strict = true)
+        super(name, ::Integer, default, strict = true) { |value| valid?(value) }
       end
 
       def valid?(value)
@@ -651,7 +696,7 @@ module LogStash
     class TimeValue < Coercible
       include LogStash::Util::Loggable
 
-      def initialize(name, default, strict=true, &validator_proc)
+      def initialize(name, default, strict = true, &validator_proc)
         super(name, Util::TimeValue, default, strict, &validator_proc)
       end
 
@@ -669,7 +714,7 @@ module LogStash
     end
 
     class ArrayCoercible < Coercible
-      def initialize(name, klass, default, strict=true, &validator_proc)
+      def initialize(name, klass, default, strict = true, &validator_proc)
         @element_class = klass
         super(name, ::Array, default, strict, &validator_proc)
       end
@@ -697,7 +742,7 @@ module LogStash
     class SplittableStringArray < ArrayCoercible
       DEFAULT_TOKEN = ","
 
-      def initialize(name, klass, default, strict=true, tokenizer = DEFAULT_TOKEN, &validator_proc)
+      def initialize(name, klass, default, strict = true, tokenizer = DEFAULT_TOKEN, &validator_proc)
         @element_class = klass
         @token = tokenizer
         super(name, klass, default, strict, &validator_proc)
@@ -714,6 +759,26 @@ module LogStash
       end
     end
 
+    class StringArray < ArrayCoercible
+      def initialize(name, default, strict = true, possible_strings = [], &validator_proc)
+        @possible_strings = possible_strings
+        super(name, ::String, default, strict, &validator_proc)
+      end
+
+      protected
+
+      def validate(value)
+        super(value)
+        return unless @possible_strings&.any?
+
+        invalid_value = coerce(value).reject { |val| @possible_strings.include?(val) }
+        return unless invalid_value.any?
+
+        raise ArgumentError,
+          "Failed to validate the setting \"#{@name}\" value(s): #{invalid_value.inspect}. Valid options are: #{@possible_strings.inspect}"
+      end
+    end
+
     class Modules < Coercible
       def initialize(name, klass, default = nil)
         super(name, klass, default, false)
@@ -760,7 +825,6 @@ module LogStash
     # to be used exclusively through `SettingWithDeprecatedAlias#wrap`
     class DeprecatedAlias < SimpleDelegator
       # include LogStash::Util::Loggable
-
       alias_method :wrapped, :__getobj__
       attr_reader :canonical_proxy
 
@@ -776,15 +840,15 @@ module LogStash
 
       def set(value)
         deprecation_logger.deprecated(I18n.t("logstash.settings.deprecation.set",
-                                             deprecated_alias: name,
-                                             canonical_name: canonical_proxy.name))
+                                             :deprecated_alias => name,
+                                             :canonical_name => canonical_proxy.name))
         super
       end
 
       def value
         logger.warn(I18n.t("logstash.settings.deprecation.queried",
-                           deprecated_alias: name,
-                           canonical_name: canonical_proxy.name))
+                           :deprecated_alias => name,
+                           :canonical_name => canonical_proxy.name))
         @canonical_proxy.value
       end
 
@@ -850,8 +914,8 @@ module LogStash
       def validate_value
         if deprecated_alias.set? && canonical_setting.set?
           fail(ArgumentError, I18n.t("logstash.settings.deprecation.ambiguous",
-                                     canonical_name: canonical_setting.name,
-                                     deprecated_alias: deprecated_alias.name))
+                                     :canonical_name => canonical_setting.name,
+                                     :deprecated_alias => deprecated_alias.name))
         end
 
         super
@@ -859,6 +923,5 @@ module LogStash
     end
   end
 
-
   SETTINGS = Settings.new
 end
